'' =================================================================================================
''
''   File....... jm_pcd8544.spin
''   Purpose.... PCD8544 LCD driver object; interface in PASM for speed
''               -- based on code by Sylvain Bissonnette
''   Author..... Jon "JonnyMac" McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 18 NOV 2012
''
'' =================================================================================================

{
  Example product:
  -- https://www.adafruit.com/products/338

  Driver chip documentation:
  -- http://www.adafruit.com/datasheets/pcd8544.pdf
}


con

  RX1  = 31                                                     ' programming port
  TX1  = 30
  
  SDA  = 29                                                     ' eeprom
  SCL  = 28


con

  LCD_CMD   =  0
  LCD_DATA  =  1

  NORMAL    = $00
  INVERT    = $FF

  LCD_X_RES = 84
  LCD_Y_RES = 48

  DDRAM_SZ  = LCD_X_RES * LCD_Y_RES / 8

  DC_BIT    = %01_00000                                         ' 0 = cmd, 1 = data
  INV_BIT   = %10_00000                                         ' 0 = normal, 1 = inverted

  FONT_SIZE = 5                                                 ' active pixels in font
  
  #0, DISP_BLANK, DISP_ALL, DISP_NORMAL, DISP_INVERT


con

  PWM_FREQ = 375 { max }                                        ' desired PWM frequency
  

var

  long  cog

  long  command                                                 ' command / parameters

  long  rst                                                     ' pcd8544 control pins
  long  sce                                                     
  long  dc                                                      
  long  sclk
  long  sdin
  
  long  led                                                     ' backlight control pin
  
  long  pwmtix                                                  ' loop timing for pwm
  long  pwmsetting                                              ' pwm setting for ctrx

  long  chrmapspntr                                             ' pointer to character maps

  long  wrinverted                                              ' 0 = normal, !0 = inverted
  

pub start(rstpin, ledpin)

'' Start PCD8544 driver
'' -- uses five contiguous pins (see order in startx method)
'' -- (optional) led control may be non-contiguous
'' -- if no LED control, use -1

  return startx(rstpin+0, rstpin+1, rstpin+2, rstpin+3, rstpin+4, ledpin)


pub startx(rstpin, scepin, dcpin, sclkpin, sdinpin, ledpin) 

'' Start PCD8544 driver using specific pins
'' -- if no LED control, use -1

  stop

  longmove(@rst, @rstpin, 6)                                    ' copy pins

  if (ledpin => 0)
    pwmtix := clkfreq / PWM_FREQ                                ' setup for ctrx pwm
    backlight(100)                                              ' full bright

  chrmapspntr := @Ascii                                         ' address of character table

  cog := cognew(@pcd8544, @command) + 1                         ' start the PASM cog

  if (cog)                                                      ' if cog loaded
    reset                                                       '  reset LCD controller
    wr_block(@Setup, 6, LCD_CMD)                                '  write setup sequence
    clear                                                       '  clear lcd
     
  return cog

    
pub stop

'' Stop PCD8544 driver if running

  if (cog)
    cogstop(cog - 1)
    cog := 0


pub reset

'' Generate reset pulse

  run_command(1)


pub set_style(mode)

'' Sets future data writes to normal or inverted mode

  if (mode)
    wrinverted := true
  else
    wrinverted := false    

  
pub contrast(value)

'' Allows adjustment of LCD contrast

  value &= $7F

  write($21, LCD_CMD)                                           ' function set (H = 1)
  write($80 | value, LCD_CMD)                                   ' set Vop (contrast)
  write($20, LCD_CMD)                                           ' function set (H = 0)

  
pub clear | idx

'' Clear LCD
'' -- leaves cursor at 0, 0

  if (wrinverted)
    wr_count($FF, DDRAM_SZ)                                     ' black pixels
  else
    wr_count($00, DDRAM_SZ)                                     ' clear pixels

  goto_xy(0, 0)


pub clearln(line)

'' Clears specific line of LCD
'' -- leaves cursor at beginning of line

  if ((line < 0) or (line > 5))                                 ' validate line
    return

  goto_xy(0, line)                                              ' move to start of line
  
  if (wrinverted)
    wr_count($FF, LCD_X_RES)                                    ' fill pixels
  else
    wr_count($00, LCD_X_RES)                                    ' clear pixels

  goto_xy(0, line)                                              ' back to start
  

pub putc(c) 

'' Write ASCII character c to LCD
'' -- uses built-in, 5x8 font; adds right-side pad (1 pixel)
'' -- see mapping of command value in PASM section

  if ((c < $20) or (c > $7F))                                   ' limit to ASCII set
    return
        
  c <<= 16                                                      ' move c to byte2
  c |= 1 << 7                                                   ' write once

  if (wrinverted)
    c |= INV_BIT

  run_command(c | 5)


pub putcx(c, count)

'' Write ASCII character c to LCD count times
'' -- uses built-in, 5x8 font; adds right-side pad (1 pixel)

  if ((c < $20) or (c > $7F) or (count < 0) )                   ' validate
    return

  c <<= 16                                                      ' move c to byte2
  count <#= constant(DDRAM_SZ / 6)                              ' limit to screen size 
  c |= count << 7                                               ' write count times  

  if (wrinverted)
    c |= INV_BIT

  run_command(c | 5)


pub str(spntr)

'' Write z-string (at spntr) to LCD

  spntr <<= 16                                                  ' move pointer to byte2
  if (wrinverted)
    spntr |= INV_BIT

  run_command(spntr | 6)


pub dec(value) | i, x               
                                      
'' Print a decimal number
'' -- borrowed from FullDuplexSerial
                                      
  x := value == negx                  
  if value < 0                        
    value := ||(value+x)              
    putc("-")                           
                                      
  i := 1_000_000_000                  
                                      
  repeat 10                           
    if value => i                     
      putc(value / i + "0" + x*(i == 1))
      value //= i                     
      result := true                        
    elseif ((result) or (i == 1))         
      putc("0")                         
    i /= 10


pub rjdec(val, width, pchar) | tmpval, pad

'' Print right-justified decimal value
'' -- val is value to print
'' -- width is width of (space padded) field for value
'' -- pad is leading character

'  Original code by Dave Hein
'  Modifications by Jon McPhalen

  if (val => 0)                                           
    tmpval := val                                         
    pad := width - 1                                      
  else
    if (val == negx)                                      
      tmpval := posx                                      
    else                                                  
      tmpval := -val                                      
    pad := width - 2                                      
   
  repeat while (tmpval => 10)                             
    pad--
    tmpval /= 10
     
  repeat pad                                              
    putc(pchar)

  dec(val)


pub hex(value, digits)

'' Print a hexadecimal number
'' -- borrowed from FullDuplexSerial

  value <<= (8 - digits) << 2
  repeat digits
    putc(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))


pub bin(value, digits)

'' Print a binary number
'' -- borrowed from FullDuplexSerial 

  value <<= 32 - digits
  repeat digits
    putc((value <-= 1) & 1 + "0")                                              

  
pub put_glyph(pntr, size)

'' Write glyph (or chacter) map (at pntr) to LCD
'' -- size is pixel width of glyph
''    * data should include pad column for characters

  wr_block(pntr, size, LCD_DATA)                                ' write the glyph block


pub goto_xy(x, y)

'' Move to specified pixel postion on line
'' -- x range is 0 to 83
'' -- y range is 0 to 5

  if ((x < 0) or (x => LCD_X_RES))                              ' validate x
    return

  if ((y < 0) or (y > 5))                                       ' validate y
    return

  write($80 | x, LCD_CMD)                                       ' move lcd cursor
  write($40 | y, LCD_CMD)

   
pub char_xy(x, y)

'' Move to specific character position
'' -- x range is 0 to 13 (internal font)
'' -- y range is 0 to 5

  if ((x < 0) or (x => (LCD_X_RES / (FONT_SIZE + 1))))          ' validate x
    return

  if ((y < 0) or (y > 5))                                       ' validate y
    return

  goto_xy(x * (FONT_SIZE + 1), y)                               ' move the cursor


pub center(spntr, y) | x

'' Center string at spntr on line y (0 to 5)

  x := (LCD_X_RES - strsize(spntr) * (FONT_SIZE + 1)) >> 1      ' calc leading gap
  goto_xy(x, y)                                                 ' move cursor
  str(spntr)                                                    ' write the string


pub backlight(level)

'' If defined, set LED brightnes to level, 0 to 100%
'' -- converted for fixed frequency PWM using ctrx in NCO mode

  if (led => 0)                                                 ' if LED used
    pwmsetting := -(pwmtix * (0 #> level <# 100) / 100)         ' convert level for NCO mode    


con

pub write(b, mode) | cp

'' Write raw byte b to LCD
'' -- mode is 0 for command, 1 for data

  cp := b << 16
  if (mode)
    cp |= DC_BIT

  run_command(cp | 2)


pub wr_count(b, count) | cp

'' Write byte count times (data mode)
'' -- used to fill area of DDRAM with byte b
'' -- count is number of times to write the byte

  cp := b << 16
  cp |= (count <# DDRAM_SZ) << 7

  run_command(cp | 3)


pub wr_block(pntr, count, mode) | cp

'' Write block of bytes to lcd
'' -- pntr is address of bytes
'' -- count is number of bytes in block
'' -- mode is 0 for command, 1 for data

  cp := pntr << 16
  cp |= (count <# DDRAM_SZ) << 7

  if ((wrinverted) and (mode))
    cp |= INV_BIT

  if (mode)
    cp |= DC_BIT

  run_command(cp | 4)


pub display_ctrl(mode)

'' Set display control register

  case mode
    DISP_BLANK:                                                 ' blank display
      write(%0000_1000, LCD_CMD)

    DISP_ALL:                                                   ' all bits on
      write(%0000_1001, LCD_CMD)   

    DISP_NORMAL:                                                ' show display
      write(%0000_1100, LCD_CMD)

    DISP_INVERT:                                                ' invert displayed bits
      write(%0000_1101, LCD_CMD)  
  

con

pri run_command(c)

'' Passes bit-packed command to PASM cog
'' -- waits for command to finish before returning

  command := c
  repeat while (command)
 

dat

Setup                   byte    $21                             ' function set (H = 1)
                        byte    $13                             ' bias system          
                        byte    $80 | $3E                       ' set Vop (contrast)
                        byte    $06                             ' temperature control
                        byte    $20                             ' function set (H = 0)
                        byte    $0C                             ' display control, normal 


' Standard 5x8 character map

Ascii                   byte    $00, $00, $00, $00, $00         ' 20  
                        byte    $00, $00, $5f, $00, $00         ' 21 !
                        byte    $00, $07, $00, $07, $00         ' 22 "
                        byte    $14, $7f, $14, $7f, $14         ' 23 #
                        byte    $24, $2a, $7f, $2a, $12         ' 24 $
                        byte    $23, $13, $08, $64, $62         ' 25 %
                        byte    $36, $49, $55, $22, $50         ' 26 &
                        byte    $00, $05, $03, $00, $00         ' 27 '
                        byte    $00, $1c, $22, $41, $00         ' 28 (
                        byte    $00, $41, $22, $1c, $00         ' 29 )
                        byte    $14, $08, $3e, $08, $14         ' 2a *
                        byte    $08, $08, $3e, $08, $08         ' 2b +
                        byte    $00, $50, $30, $00, $00         ' 2c ,
                        byte    $08, $08, $08, $08, $08         ' 2d -
                        byte    $00, $60, $60, $00, $00         ' 2e .
                        byte    $20, $10, $08, $04, $02         ' 2f /
                        byte    $3e, $51, $49, $45, $3e         ' 30 0
                        byte    $00, $42, $7f, $40, $00         ' 31 1
                        byte    $42, $61, $51, $49, $46         ' 32 2
                        byte    $21, $41, $45, $4b, $31         ' 33 3
                        byte    $18, $14, $12, $7f, $10         ' 34 4
                        byte    $27, $45, $45, $45, $39         ' 35 5
                        byte    $3c, $4a, $49, $49, $30         ' 36 6
                        byte    $01, $71, $09, $05, $03         ' 37 7
                        byte    $36, $49, $49, $49, $36         ' 38 8
                        byte    $06, $49, $49, $29, $1e         ' 39 9
                        byte    $00, $36, $36, $00, $00         ' 3a :
                        byte    $00, $56, $36, $00, $00         ' 3b ;
                        byte    $08, $14, $22, $41, $00         ' 3c <
                        byte    $14, $14, $14, $14, $14         ' 3d =
                        byte    $00, $41, $22, $14, $08         ' 3e >
                        byte    $02, $01, $51, $09, $06         ' 3f ?
                        byte    $32, $49, $79, $41, $3e         ' 40 @
                        byte    $7e, $11, $11, $11, $7e         ' 41 A
                        byte    $7f, $49, $49, $49, $36         ' 42 B
                        byte    $3e, $41, $41, $41, $22         ' 43 C
                        byte    $7f, $41, $41, $22, $1c         ' 44 D
                        byte    $7f, $49, $49, $49, $41         ' 45 E
                        byte    $7f, $09, $09, $09, $01         ' 46 F
                        byte    $3e, $41, $49, $49, $7a         ' 47 G
                        byte    $7f, $08, $08, $08, $7f         ' 48 H
                        byte    $00, $41, $7f, $41, $00         ' 49 I
                        byte    $20, $40, $41, $3f, $01         ' 4a J
                        byte    $7f, $08, $14, $22, $41         ' 4b K
                        byte    $7f, $40, $40, $40, $40         ' 4c L
                        byte    $7f, $02, $0c, $02, $7f         ' 4d M
                        byte    $7f, $04, $08, $10, $7f         ' 4e N
                        byte    $3e, $41, $41, $41, $3e         ' 4f O
                        byte    $7f, $09, $09, $09, $06         ' 50 P
                        byte    $3e, $41, $51, $21, $5e         ' 51 Q
                        byte    $7f, $09, $19, $29, $46         ' 52 R
                        byte    $46, $49, $49, $49, $31         ' 53 S
                        byte    $01, $01, $7f, $01, $01         ' 54 T
                        byte    $3f, $40, $40, $40, $3f         ' 55 U
                        byte    $1f, $20, $40, $20, $1f         ' 56 V
                        byte    $3f, $40, $38, $40, $3f         ' 57 W
                        byte    $63, $14, $08, $14, $63         ' 58 X
                        byte    $07, $08, $70, $08, $07         ' 59 Y
                        byte    $61, $51, $49, $45, $43         ' 5a Z
                        byte    $00, $7f, $41, $41, $00         ' 5b [
                        byte    $02, $04, $08, $10, $20         ' 5c 
                        byte    $00, $41, $41, $7f, $00         ' 5d ]
                        byte    $04, $02, $01, $02, $04         ' 5e ^
                        byte    $40, $40, $40, $40, $40         ' 5f _
                        byte    $00, $01, $02, $04, $00         ' 60 `
                        byte    $20, $54, $54, $54, $78         ' 61 a
                        byte    $7f, $48, $44, $44, $38         ' 62 b
                        byte    $38, $44, $44, $44, $20         ' 63 c
                        byte    $38, $44, $44, $48, $7f         ' 64 d
                        byte    $38, $54, $54, $54, $18         ' 65 e
                        byte    $08, $7e, $09, $01, $02         ' 66 f
                        byte    $0c, $52, $52, $52, $3e         ' 67 g
                        byte    $7f, $08, $04, $04, $78         ' 68 h
                        byte    $00, $44, $7d, $40, $00         ' 69 i
                        byte    $20, $40, $44, $3d, $00         ' 6a j 
                        byte    $7f, $10, $28, $44, $00         ' 6b k
                        byte    $00, $41, $7f, $40, $00         ' 6c l
                        byte    $7c, $04, $18, $04, $78         ' 6d m
                        byte    $7c, $08, $04, $04, $78         ' 6e n
                        byte    $38, $44, $44, $44, $38         ' 6f o
                        byte    $7c, $14, $14, $14, $08         ' 70 p
                        byte    $08, $14, $14, $18, $7c         ' 71 q
                        byte    $7c, $08, $04, $04, $08         ' 72 r
                        byte    $48, $54, $54, $54, $20         ' 73 s
                        byte    $04, $3f, $44, $40, $20         ' 74 t
                        byte    $3c, $40, $40, $20, $7c         ' 75 u
                        byte    $1c, $20, $40, $20, $1c         ' 76 v
                        byte    $3c, $40, $30, $40, $3c         ' 77 w
                        byte    $44, $28, $10, $28, $44         ' 78 x
                        byte    $0c, $50, $50, $50, $3c         ' 79 y
                        byte    $44, $64, $54, $4c, $44         ' 7a z
                        byte    $00, $08, $36, $41, $00         ' 7b {
                        byte    $00, $00, $7f, $00, $00         ' 7c |
                        byte    $00, $41, $36, $08, $00         ' 7d }
                        byte    $10, $08, $08, $10, $08         ' 7e ~
                        byte    $78, $46, $41, $46, $78         ' 7f 
  

dat

' Command long is bit-packed with parameter(s)
'
'  4...0 -> Command 
'      5 -> DC bit
'      6 -> 0 = normal write, 1 = inverted write (data mode only)
' 15...7 -> byte count, 0-511
' 31..16 -> byte (to write) or hub address (blocks and strings)

                        org     0

pcd8544                 mov     dira, #0                        ' clear pins
                        mov     outa, #0
                        mov     t1, par                         ' start of structure

                        add     t1, #4
                        rdlong  t2, t1                          ' read RST pin
                        mov     rstmask, #1                     ' create pin mask
                        shl     rstmask, t2
                        or      outa, rstmask                   ' start high
                        or      dira, rstmask                   ' set to output
                                                
                        add     t1, #4      
                        rdlong  t2, t1    
                        mov     scemask, #1                     ' read SCE (CS) pin
                        shl     scemask, t2
                        or      outa, scemask                   ' deselect
                        or      dira, scemask 

                        add     t1, #4      
                        rdlong  t2, t1                          ' read DC pin   
                        mov     dcmask, #1   
                        shl     dcmask, t2 
                        or      dira, dcmask

                        add     t1, #4      
                        rdlong  t2, t1                          ' read SCK pin   
                        mov     sckmask, #1   
                        shl     sckmask, t2 
                        or      dira, sckmask

                        add     t1, #4      
                        rdlong  t2, t1                          ' read MOSI (DIN) pin   
                        mov     mosimask, #1   
                        shl     mosimask, t2 
                        or      dira, mosimask

                        ' setup ctra for high-speed shiftout

                        add     t2, NCO_SE                      ' add NCO mode to pin #
                        mov     frqa, #0                        ' do not auto increment
                        mov     phsa, #0                        ' clear to start
                        mov     ctra, t2                        ' use MOSI pin for counter output

                        add     t1, #4
                        rdlong  t2, t1                          ' read LED pin
                        cmps    t2, #0                  wc, wz  ' led used?
        if_b            mov     ledmask, #0                     ' no, clear mask
        if_b            add     t1, #8                          '  skip over next two
        if_b            jmp     #getmaps
        
                        mov     ledmask, #1                     ' make led mask
                        shl     ledmask, t2
                        or      dira, ledmask                   ' set to output

                        add     t1, #4                          
                        rdlong  looptix, t1                     ' read loop timing (ticks)

                        add     t1, #4
                        mov     levelhub, t1                    ' copy hub address of led level
                        
                        ' setup ctrb for pwm
                         
                        add     t2, NCO_SE                      ' add NCO mode to pin #
                        mov     frqb, #1
                        mov     ctrb, t2                        ' use LED pin as pwm output
        
getmaps                 add     t1, #4
                        rdlong  hubcharmaps, t1                 ' get hub address of char maps
                        
starttimer              mov     looptimer, cnt
                        add     looptimer, looptix

                        jmp     #cmdexit

                                                

getcommand              rdlong  cmd, par
                        mov     uparms, cmd                     ' save parameters
                        and     cmd, #$1F               wz      ' strip to command
        if_z            jmp     #updateled                      ' process pwm

                        max     cmd, #7                         ' available commands + 1
                        add     cmd, #jumptable
                        jmp     cmd

jumptable               jmp     #cmdexit                        ' cmd = 0
                        jmp     #cmd_reset                      ' cmd = 1
                        jmp     #cmd_wr_byte                    ' cmd = 2
                        jmp     #cmd_wr_count                   ' cmd = 3
                        jmp     #cmd_wr_block                   ' cmd = 4
                        jmp     #cmd_wr_char                    ' cmd = 5
                        jmp     #cmd_wr_str                     ' cmd = 6

cmdexit                 or      outa, scemask                   ' deselect lcd
                        mov     cmd, #0
                        wrlong  cmd, par

updateled               tjz     ledmask, #getcommand            ' back to top if no led    

                        waitcnt looptimer, looptix              ' let loop timing finish                
                        rdlong  phsb, levelhub                  ' if used, update brightness
                        jmp     #getcommand   

                        

cmd_reset               andn    outa, rstmask                   ' rst pin low
                        rdlong  t1, #0                          ' read tics/sec
                        shr     t1, #10                         ' ~1ms
                        add     t1, cnt                         ' sync
                        waitcnt t1, #0
                        or      outa, rstmask                   ' rst pin high
                        jmp     #cmdexit

                        
                        
cmd_wr_byte             andn    outa, scemask                   ' select
                        test    uparms, #DC_BIT         wc      ' check dc bit
                        muxc    outa, dcmask                    ' update dc pin
                        mov     outbyte, uparms                 ' get byte
                        shr     outbyte, #16                    ' extract 
                        call    #shiftout
                        jmp     #cmdexit

                        
                                                
cmd_wr_count            andn    outa, scemask                   ' select
                        or      outa, dcmask                    ' set to data mode
                        mov     t1, uparms                      ' copy byte
                        shr     t1, #16                         ' extract
                        mov     t2, uparms                      ' copy count
                        shr     t2, #7                          ' extract
                        and     t2, #$1FF                       ' cleanup (remove byte)
:loop                   mov     outbyte, t1                     ' copy output
                        call    #shiftout                       ' write to display
                        djnz    t2, #:loop                      ' decrement count, exit on 0
                        jmp     #cmdexit

                        

cmd_wr_block            andn    outa, scemask                   ' select
                        test    uparms, #DC_BIT         wc      ' check dc bit
                        muxc    outa, dcmask                    ' update dc pin 
                        mov     t1, uparms                      ' copy address
                        shr     t1, #16                         ' extract
                        mov     t2, uparms                      ' copy count
                        shr     t2, #7
                        and     t2, #$1FF                       ' cleanup (remove address)
                        test    uparms, #INV_BIT        wz      ' if nz, inverted bytes
:loop                   rdbyte  outbyte, t1                     ' get byte from block
        if_nz           xor     outbyte, #$FF                   ' invert if INV_BIT = 1                 
                        call    #shiftout
                        add     t1, #1                          ' update pointer
                        djnz    t2, #:loop
                        jmp     #cmdexit


                        
cmd_wr_char             andn    outa, scemask                   ' select   
                        or      outa, dcmask                    ' set to data mode
                        mov     t3, uparms                      ' copy char
                        shr     t3, #16                         ' extract
                        mov     t4, uparms                      ' get count
                        shr     t4, #7                          ' extract
                        and     t4, #$7F                        ' cleanup
:loop                   mov     t1, t3                          ' char out in t1
                        call    #charout                        ' send to display
                        djnz    t4, #:loop
                        jmp     #cmdexit


                        
cmd_wr_str              andn    outa, scemask                   ' select
                        or      outa, dcmask                    ' set to data mode
                        mov     t3, uparms                      ' copy string address
                        shr     t3, #16                         ' extract
:loop                   rdbyte  t1, t3                  wz      ' get a character
        if_z            jmp     #cmdexit                        ' abort on 0
                        call    #charout                        ' write the character
                        add     t3, #1                          ' point to next char
                        jmp     #:loop    



' Writes ASCII character in t1 to LCD
' -- valid characters from $20 to $7F

charout                 sub     t1, #$20                        ' convert to index (0..n)
                        mov     chidx, #0
                        mov     fwidth, #FONT_SIZE              ' active pixels in font
:mult                   add     chidx, t1                       ' chidx := t1 x width
                        djnz    fwidth, #:mult
                        add     chidx, hubcharmaps              ' point into char map      
                        mov     fwidth, #5
                        test    uparms, #INV_BIT        wz      ' if nz, inverted bytes 
:loop                   rdbyte  outbyte, chidx
        if_nz           xor     outbyte, #$FF                   ' invert if INV_BIT = 1                 
                        call    #shiftout
                        add     chidx, #1                       ' increment pointer
                        djnz    fwidth, #:loop
        if_z            mov     outbyte, #0                     ' right side pad
        if_nz           mov     outbyte, #$FF
                        call    #shiftout
charout_ret             ret                        
                        


' Shifts byte in outbyte to LCD 

shiftout                shl     outbyte, #(32 - 8)              ' move MSB to bit31  
                        mov     phsa, outbyte                   ' copy for output
                        mov     bits, #8                        ' shift 8 bits
:loop                   or      outa, sckmask                   ' sck high
                        shl     phsa, #1                        ' get next bit
                        andn    outa, sckmask                   ' sck low
                        djnz    bits, #:loop
shiftout_ret            ret


' ----------------------------------------------------------------------------------------

NCO_SE                  long    %00100 << 26                    ' for counters

rstmask                 res     1                               ' pin mask for RST
scemask                 res     1                               ' pin mask for SCE
dcmask                  res     1                               ' pin mask for DC
sckmask                 res     1                               ' pin mask for SCK
mosimask                res     1                               ' pin mask for MOSI
ledmask                 res     1                               ' pin mask for LED
looptix                 res     1                               ' loop timing
levelhub                res     1                               ' hub addres of led level
hubcharmaps             res     1                               ' hub address of char maps

looptimer               res     1                               ' timer for pwm loop

cmd                     res     1                               ' command
uparms                  res     1                               ' parameters for command

outbyte                 res     1                               ' for shiftout
bits                    res     1

chidx                   res     1                               ' for charout
fwidth                  res     1

t1                      res     1                               ' work vars
t2                      res     1
t3                      res     1
t4                      res     1

                        fit     496
                        

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}